/*
 * Copyright 2007 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _DNSSCAN_H_
#define _DNSSCAN_H_

#define DNSSCAN_TAG_REQUEST		1

/* number of sendto's per interface_write call */
#define DNS_MAX_SEND_AT_ONCE		50

#define DNS_MAX_GENERATE_ATTEMPTS	10000
#define DNS_MAX_ADDRESSES_AT_ONCE	20000
#define DNS_MAX_PENDING_ADDRESSES	150000

#define DNS_MAX_REQUEST_RETRIES		1
#define DNS_REQUEST_TIMEOUT_IN_SEC	5

#define DNS_CHECKPOINT_INTERVAL_IN_SEC	(10 * 60)
#define DNS_ROTATE_INTERVAL_IN_SEC	(4 * 3600)
#define DNS_RERUN_INTERVAL_IN_SEC	(1 * 3600)

#define MAX_HTTP_CONNECTIONS		250
#define MAX_HTTP_PAYLOAD_SIZE		256

struct generate {
	TAILQ_ENTRY(generate) (next);

	uint32_t prefix;
	int bits;
	uint32_t last_count; /* the last count in the range */
};

/* we might have a lot of generators, support binary search on them */
struct generate_index {
	uint32_t first_count;
	struct generate *gen;
};

struct generators {
	TAILQ_HEAD(generateq, generate) (generators);

	uint32_t count;
	uint32_t last_count;
	uint32_t last_real_count;
	int bits;

	struct generate_index *index;
	size_t num_index;

	int done;	/* set when we have generated everything */
};

struct HTTPServerReply;
struct interface;
struct request {
	TAILQ_ENTRY(request) (next);

	uint16_t id;
	int retries;

	struct state *state;

	struct interface *inf;
	struct addr dest;

	/* always set */
	uint8_t *payload;
	int payload_size;

	struct event ev;	/* for timeouts */

	/* set for FPDNS */
	struct addr reply;
	int reply_set;

	char *chaos_reply;

	uint8_t *chaos_payload;
	int chaos_payload_size;

	char *fpdns_iquery;
	char *fpdns_tc;

	char *fphttp_server;
	char *fphttp_date;
	char *fphttp_powered;
	
	int fphttp_body_len;
	uint8_t *http_body_payload;
	int http_body_payload_size;
	
	/* complete log of all headers */
	struct HTTPServerReply *server_reply;

	/* for getting a web fingerprint */
	struct evhttp_connection *ev_con;
};

TAILQ_HEAD(requestq, request);

struct event_counter;
struct interface {
	TAILQ_ENTRY(interface) (next);

	char *name;
	int fd;	/* bound socket to the interface */

	struct event ev_read;
	struct event ev_write;

	struct event_counter *ctr_bytes_read;
	struct event_counter *ctr_bytes_sent;
	
	struct requestq pending;
	int num_pending;

	struct requestq inflight;
	int num_inflight;
};

TAILQ_HEAD(interfaceq, interface);

#define DFPRINTF(x, y) do { \
		extern int verbose; \
		if (verbose >= x) fprintf y; \
	} while (0)

void add_request_to_interface(struct interface *inf, struct addr *addr);

void request_log(struct request *req);
void request_free(struct request *req);

#endif /* _DNSSCAN_H_ */
