/*
 * Copyright 2007 Niels Provos <provos@citi.umich.edu>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <sys/queue.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <strings.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>

#include <event.h>
#include <dnet.h>

#include "dnsscan.h"
#include "state.h"

struct statetable *
statetable_new(int initial_state, int num_states)
{
	struct statetable *st = calloc(1, sizeof(struct statetable));
	assert(st != NULL);

	st->initial_state = initial_state;
	st->num_states = num_states;
	st->states = calloc(num_states, sizeof(struct state *));

	return (st);
}

struct state *
statetable_get(struct statetable *st, int state)
{
	assert(state >= 0 && state < st->num_states);
	return (st->states[state]);
}

void
statetable_add(struct statetable *st, int state,
    int (*probe_init)(struct state *, struct request *, uint8_t *, size_t),
    int (*next_state)(struct state *, struct request *, 
	struct addr *, uint8_t *, size_t))
{
	struct state *new_state = NULL;
	assert(state >= 0 && state < st->num_states);

	new_state = calloc(1, sizeof(struct state));
	assert(new_state != NULL);

	new_state->state = state;
	new_state->probe_init = probe_init;
	new_state->next_state = next_state;

	assert(st->states[state] == NULL);
	st->states[state] = new_state;
}

void
statetable_set_abort(struct statetable *st, void (*cb)(struct request *))
{
	st->abort_state = cb;
}

void
statetable_set_finish(struct statetable *st, void (*cb)(struct request *))
{
	st->finish_state = cb;
}
